import time
import visa
import numpy

class LxiInstr(object):

    timeout_msec = 10000

    visa_rm = visa.ResourceManager()
    trace = False

    def __init__(self, addr):
        super(LxiInstr, self).__init__()
        self.addr  = addr
        self.instr = self.visa_rm.open_resource(addr)
        idn        = self.instr.query("*IDN?")
        if idn[-1] == '\n':
            idn = idn[:-1]
        (self.instr_make, self.instr_model, self.instr_serial, self.instr_fw_ver) = idn.split(',', 3)
        self.instr.timeout = self.timeout_msec

        # pick up errors
        self.error()
        self.error()

    def oneliner(self):
        return "%s %s (FW %s) @ %s" % (self.instr_make, self.instr_model, self.instr_fw_ver, self.addr)

    def make(self): return self.instr_make
    def model(self): return self.instr_model

    def query(self, s, stripnl = True):
        if self.trace:
            print "* %s >>> '%s'" % (self.model(), s)
        r = self.instr.query(s)
        if len(r) > 0 and r[-1] == '\n':
            r = r[:-1]
        if self.trace:
            print "* %s <<< '%s'" % (self.model(), r)
        return r

    def query_ascii_values(self, s):
        if self.trace:
            print "* %s >>> '%s'" % (self.model(), s)
        r = self.instr.query_ascii_values(s)
        if self.trace:
            print "* %s <<< '%s'" % (self.model(), r)
        return numpy.array(r)

    # Check for error
    def error(self):
        s = self.query("SYST:ERR?")
        ok = 'No error' in s
        if not ok:
            print "ERROR: %s" % s
        return not ok

    # Send a command to instrument, without checking
    def send(self, command):
        if self.trace:
            print "* %s >>> '%s'" % (self.model(), command)
        self.instr.write(command)

    # Issue command and check result
    def command(self, command):
        self.send(command)
        ok = not self.error()
        if not ok:
            print "Command '%s' failed" % command
        return ok

    # RESET
    def cmd_reset(self):
        self.error()
        return self.command("*RST")
